@@ -1,4 +1,6 @@
-use crate::utils::{distance_between, slice_from_raw_parts, slice_from_raw_parts_mut};
+use crate::utils::{
+  distance_between, is_null_const, is_null_mut, slice_from_raw_parts, slice_from_raw_parts_mut,
+};
 use crate::StaticVec;
 use core::fmt::{self, Debug, Formatter};
 use core::intrinsics;
@@ -85,16 +87,22 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
 
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
-    match distance_between(self.end, self.start) {
-      0 => None,
-      _ => unsafe {
-        let res = Some(&*self.start);
-        self.start = match intrinsics::size_of::<T>() {
-          0 => (self.start as usize + 1) as *const T,
-          _ => self.start.offset(1),
-        };
-        res
-      },
+    unsafe {
+      intrinsics::assume(!is_null_const(self.start));
+      if intrinsics::size_of::<T>() != 0 {
+        intrinsics::assume(!is_null_const(self.end));
+      }
+      match distance_between(self.end, self.start) {
+        0 => None,
+        _ => {
+          let res = Some(&*self.start);
+          self.start = match intrinsics::size_of::<T>() {
+            0 => (self.start as usize + 1) as *const T,
+            _ => self.start.offset(1),
+          };
+          res
+        }
+      }
     }
   }
 
@@ -103,20 +111,73 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
     let len = distance_between(self.end, self.start);
     (len, Some(len))
   }
+
+  #[inline(always)]
+  fn count(self) -> usize {
+    self.len()
+  }
+
+  #[inline(always)]
+  fn nth(&mut self, n: usize) -> Option<Self::Item> {
+    if n >= self.len() {
+      None
+    } else {
+      unsafe {
+        match intrinsics::size_of::<T>() {
+          0 => {
+            let res = (self.start as usize + n) as *const T;
+            self.start = (res as usize + 1) as *const T;
+            Some(&*res)
+          }
+          _ => {
+            let res = self.start.add(n);
+            self.start = res.offset(1);
+            Some(&*res)
+          }
+        }
+      }
+    }
+  }
+
+  #[inline(always)]
+  fn last(mut self) -> Option<Self::Item> {
+    self.next_back()
+  }
 }
 
 impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterConst<'a, T, N> {
   #[inline(always)]
   fn next_back(&mut self) -> Option<Self::Item> {
-    match distance_between(self.end, self.start) {
-      0 => None,
-      _ => unsafe {
+    unsafe {
+      intrinsics::assume(!is_null_const(self.start));
+      if intrinsics::size_of::<T>() != 0 {
+        intrinsics::assume(!is_null_const(self.end));
+      }
+      match distance_between(self.end, self.start) {
+        0 => None,
+        _ => {
+          self.end = match intrinsics::size_of::<T>() {
+            0 => (self.end as usize - 1) as *const T,
+            _ => self.end.offset(-1),
+          };
+          Some(&*self.end)
+        }
+      }
+    }
+  }
+
+  #[inline(always)]
+  fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
+    if n >= self.len() {
+      None
+    } else {
+      unsafe {
         self.end = match intrinsics::size_of::<T>() {
-          0 => (self.end as usize - 1) as *const T,
-          _ => self.end.offset(-1),
+          0 => (self.end as *const u8).wrapping_offset(-((n as isize) + 1)) as *const T,
+          _ => self.end.offset(-((n as isize) + 1)),
         };
         Some(&*self.end)
-      },
+      }
     }
   }
 }
@@ -196,16 +257,22 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
 
   #[inline(always)]
   fn next(&mut self) -> Option<Self::Item> {
-    match distance_between(self.end, self.start) {
-      0 => None,
-      _ => unsafe {
-        let res = Some(&mut *self.start);
-        self.start = match intrinsics::size_of::<T>() {
-          0 => (self.start as usize + 1) as *mut T,
-          _ => self.start.offset(1),
-        };
-        res
-      },
+    unsafe {
+      intrinsics::assume(!is_null_mut(self.start));
+      if intrinsics::size_of::<T>() != 0 {
+        intrinsics::assume(!is_null_mut(self.end));
+      }
+      match distance_between(self.end, self.start) {
+        0 => None,
+        _ => {
+          let res = Some(&mut *self.start);
+          self.start = match intrinsics::size_of::<T>() {
+            0 => (self.start as usize + 1) as *mut T,
+            _ => self.start.offset(1),
+          };
+          res
+        }
+      }
     }
   }
 
@@ -214,20 +281,73 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
     let len = distance_between(self.end, self.start);
     (len, Some(len))
   }
+
+  #[inline(always)]
+  fn count(self) -> usize {
+    self.len()
+  }
+
+  #[inline(always)]
+  fn nth(&mut self, n: usize) -> Option<Self::Item> {
+    if n >= self.len() {
+      None
+    } else {
+      unsafe {
+        match intrinsics::size_of::<T>() {
+          0 => {
+            let res = (self.start as usize + n) as *mut T;
+            self.start = (res as usize + 1) as *mut T;
+            Some(&mut *res)
+          }
+          _ => {
+            let res = self.start.add(n);
+            self.start = res.offset(1);
+            Some(&mut *res)
+          }
+        }
+      }
+    }
+  }
+
+  #[inline(always)]
+  fn last(mut self) -> Option<Self::Item> {
+    self.next_back()
+  }
 }
 
 impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecIterMut<'a, T, N> {
   #[inline(always)]
   fn next_back(&mut self) -> Option<Self::Item> {
-    match distance_between(self.end, self.start) {
-      0 => None,
-      _ => unsafe {
+    unsafe {
+      intrinsics::assume(!is_null_mut(self.start));
+      if intrinsics::size_of::<T>() != 0 {
+        intrinsics::assume(!is_null_mut(self.end));
+      }
+      match distance_between(self.end, self.start) {
+        0 => None,
+        _ => {
+          self.end = match intrinsics::size_of::<T>() {
+            0 => (self.end as usize - 1) as *mut T,
+            _ => self.end.offset(-1),
+          };
+          Some(&mut *self.end)
+        }
+      }
+    }
+  }
+
+  #[inline(always)]
+  fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
+    if n >= self.len() {
+      None
+    } else {
+      unsafe {
         self.end = match intrinsics::size_of::<T>() {
-          0 => (self.end as usize - 1) as *mut T,
-          _ => self.end.offset(-1),
+          0 => (self.end as *mut u8).wrapping_offset(-((n as isize) + 1)) as *mut T,
+          _ => self.end.offset(-((n as isize) + 1)),
         };
         Some(&mut *self.end)
-      },
+      }
     }
   }
 }
@@ -247,7 +367,7 @@ impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecIterMut<'a, T, N>
 impl<'a, T: 'a, const N: usize> FusedIterator for StaticVecIterMut<'a, T, N> {}
 unsafe impl<'a, T: 'a, const N: usize> TrustedLen for StaticVecIterMut<'a, T, N> {}
 unsafe impl<'a, T: 'a + Sync, const N: usize> Sync for StaticVecIterMut<'a, T, N> {}
-unsafe impl<'a, T: 'a + Sync, const N: usize> Send for StaticVecIterMut<'a, T, N> {}
+unsafe impl<'a, T: 'a + Send, const N: usize> Send for StaticVecIterMut<'a, T, N> {}
 
 impl<'a, T: 'a + Debug, const N: usize> Debug for StaticVecIterMut<'a, T, N> {
   #[inline(always)]
@@ -324,6 +444,34 @@ impl<T, const N: usize> Iterator for StaticVecIntoIter<T, N> {
     let len = self.end - self.start;
     (len, Some(len))
   }
+
+  #[inline(always)]
+  fn count(self) -> usize {
+    self.len()
+  }
+
+  #[inline(always)]
+  fn nth(&mut self, n: usize) -> Option<Self::Item> {
+    if n >= self.len() {
+      None
+    } else {
+      unsafe {
+        let res_index = self.start + n;
+        let res = StaticVec::first_ptr(&self.data).add(res_index);
+        ptr::drop_in_place(slice_from_raw_parts_mut(
+          StaticVec::first_ptr_mut(&mut self.data).add(self.start),
+          res_index - self.start,
+        ));
+        self.start = res_index + 1;
+        Some(res.read())
+      }
+    }
+  }
+
+  #[inline(always)]
+  fn last(mut self) -> Option<Self::Item> {
+    self.next_back()
+  }
 }
 
 impl<T, const N: usize> DoubleEndedIterator for StaticVecIntoIter<T, N> {
@@ -337,6 +485,24 @@ impl<T, const N: usize> DoubleEndedIterator for StaticVecIntoIter<T, N> {
       }
     }
   }
+
+  #[inline(always)]
+  fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
+    if n >= self.len() {
+      None
+    } else {
+      let old_end = self.end;
+      let res_index = old_end - n;
+      self.end = res_index - 1;
+      unsafe {
+        ptr::drop_in_place(slice_from_raw_parts_mut(
+          StaticVec::first_ptr_mut(&mut self.data).add(res_index),
+          old_end - res_index,
+        ));
+        Some(StaticVec::first_ptr(&self.data).add(self.end).read())
+      }
+    }
+  }
 }
 
 impl<T, const N: usize> ExactSizeIterator for StaticVecIntoIter<T, N> {
@@ -354,7 +520,18 @@ impl<T, const N: usize> ExactSizeIterator for StaticVecIntoIter<T, N> {
 impl<T, const N: usize> FusedIterator for StaticVecIntoIter<T, N> {}
 unsafe impl<T, const N: usize> TrustedLen for StaticVecIntoIter<T, N> {}
 unsafe impl<T: Sync, const N: usize> Sync for StaticVecIntoIter<T, N> {}
-unsafe impl<T: Sync, const N: usize> Send for StaticVecIntoIter<T, N> {}
+unsafe impl<T: Send, const N: usize> Send for StaticVecIntoIter<T, N> {}
+
+impl<T: Clone, const N: usize> Clone for StaticVecIntoIter<T, N> {
+  #[inline(always)]
+  fn clone(&self) -> StaticVecIntoIter<T, N> {
+    let mut res = StaticVec::new();
+    for item in self.as_slice() {
+      unsafe { res.push_unchecked(item.clone()) };
+    }
+    res.into_iter()
+  }
+}
 
 impl<T: Debug, const N: usize> Debug for StaticVecIntoIter<T, N> {
   #[inline(always)]
@@ -417,6 +594,16 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecDrain<'a, T, N> {
   fn size_hint(&self) -> (usize, Option<usize>) {
     self.iter.size_hint()
   }
+
+  #[inline(always)]
+  fn count(self) -> usize {
+    self.len()
+  }
+
+  #[inline(always)]
+  fn last(mut self) -> Option<Self::Item> {
+    self.next_back()
+  }
 }
 
 impl<'a, T: 'a, const N: usize> DoubleEndedIterator for StaticVecDrain<'a, T, N> {
@@ -444,7 +631,7 @@ impl<'a, T: 'a, const N: usize> ExactSizeIterator for StaticVecDrain<'a, T, N> {
 impl<'a, T: 'a, const N: usize> FusedIterator for StaticVecDrain<'a, T, N> {}
 unsafe impl<'a, T: 'a, const N: usize> TrustedLen for StaticVecDrain<'a, T, N> {}
 unsafe impl<'a, T: 'a + Sync, const N: usize> Sync for StaticVecDrain<'a, T, N> {}
-unsafe impl<'a, T: 'a + Sync, const N: usize> Send for StaticVecDrain<'a, T, N> {}
+unsafe impl<'a, T: 'a + Send, const N: usize> Send for StaticVecDrain<'a, T, N> {}
 
 impl<'a, T: 'a + Debug, const N: usize> Debug for StaticVecDrain<'a, T, N> {
   #[inline(always)]
