@@ -27,11 +27,15 @@
 #![feature(untagged_unions)]
 
 pub use crate::errors::{CapacityError, PushCapacityError};
+pub use crate::heap::{
+  StaticHeap, StaticHeapDrain, StaticHeapDrainSorted, StaticHeapIntoIter, StaticHeapIter,
+  StaticHeapPeekMut,
+};
 pub use crate::iterators::*;
-pub use crate::string::string_utils;
-pub use crate::string::{StaticString, StringError};
+pub use crate::string::{string_utils, StaticString, StringError};
 use crate::utils::{
-  quicksort_internal, reverse_copy, slice_from_raw_parts, slice_from_raw_parts_mut,
+  is_null_const, is_null_mut, quicksort_internal, reverse_copy, slice_from_raw_parts,
+  slice_from_raw_parts_mut,
 };
 use core::cmp::{Ord, PartialEq};
 use core::intrinsics;
@@ -62,8 +66,10 @@ mod macros;
 #[doc(hidden)]
 mod errors;
 #[doc(hidden)]
+mod heap;
+#[doc(hidden)]
 mod string;
-mod trait_impls;
+pub(crate) mod trait_impls;
 #[doc(hidden)]
 pub mod utils;
 
@@ -155,7 +161,10 @@ impl<T, const N: usize> StaticVec<T, N> {
               .as_ptr()
               .copy_to_nonoverlapping(Self::first_ptr_mut(&mut data), N2.min(N));
             // Wrap the values in a MaybeUninit to inhibit their destructors (if any),
-            // then manually drop any excess ones.
+            // then manually drop any excess ones. From the assembly output I've looked
+            // at, the compiler interprets this whole sequence in a way that doesn't result
+            // in any excess copying, so there should be no performance concerns for larger
+            // input arrays.
             let mut forgotten = MaybeUninit::new(values);
             ptr::drop_in_place(forgotten.get_mut().get_unchecked_mut(N2.min(N)..N2));
             data
@@ -278,12 +287,18 @@ impl<T, const N: usize> StaticVec<T, N> {
   }
 
   /// Returns a constant pointer to the first element of the StaticVec's internal array.
+  /// It is up to the caller to ensure that the StaticVec lives for as long as they intend
+  /// to make use of the returned pointer, as once the StaticVec is dropped the pointer will
+  /// point to uninitialized or "garbage" memory.
   #[inline(always)]
   pub const fn as_ptr(&self) -> *const T {
     Self::first_ptr(&self.data)
   }
 
   /// Returns a mutable pointer to the first element of the StaticVec's internal array.
+  /// It is up to the caller to ensure that the StaticVec lives for as long as they intend
+  /// to make use of the returned pointer, as once the StaticVec is dropped the pointer will
+  /// point to uninitialized or "garbage" memory.
   #[inline(always)]
   pub const fn as_mut_ptr(&mut self) -> *mut T {
     Self::first_ptr_mut(&mut self.data)
@@ -475,9 +490,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   #[inline(always)]
   pub fn try_push(&mut self, value: T) -> Result<(), PushCapacityError<T, N>> {
     if self.is_not_full() {
-      unsafe {
-        self.push_unchecked(value);
-      };
+      unsafe { self.push_unchecked(value) };
       Ok(())
     } else {
       Err(PushCapacityError::new(value))
@@ -705,9 +718,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Removes all contents from the StaticVec and sets its length back to 0.
   #[inline(always)]
   pub fn clear(&mut self) {
-    unsafe {
-      ptr::drop_in_place(self.as_mut_slice());
-    }
+    unsafe { ptr::drop_in_place(self.as_mut_slice()) };
     self.length = 0;
   }
 
@@ -715,13 +726,17 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// inhabited area.
   #[inline(always)]
   pub fn iter(&self) -> StaticVecIterConst<T, N> {
-    StaticVecIterConst {
-      start: self.as_ptr(),
-      end: match intrinsics::size_of::<T>() {
-        0 => (self.as_ptr() as *const u8).wrapping_add(self.length) as *const T,
-        _ => unsafe { self.ptr_at_unchecked(self.length) },
-      },
-      marker: PhantomData,
+    let start_ptr = self.as_ptr();
+    unsafe {
+      intrinsics::assume(!is_null_const(start_ptr));
+      StaticVecIterConst {
+        start: start_ptr,
+        end: match intrinsics::size_of::<T>() {
+          0 => (start_ptr as *const u8).wrapping_add(self.length) as *const T,
+          _ => start_ptr.add(self.length),
+        },
+        marker: PhantomData,
+      }
     }
   }
 
@@ -729,13 +744,17 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// inhabited area.
   #[inline(always)]
   pub fn iter_mut(&mut self) -> StaticVecIterMut<T, N> {
-    StaticVecIterMut {
-      start: self.as_mut_ptr(),
-      end: match intrinsics::size_of::<T>() {
-        0 => (self.as_mut_ptr() as *mut u8).wrapping_add(self.length) as *mut T,
-        _ => unsafe { self.mut_ptr_at_unchecked(self.length) },
-      },
-      marker: PhantomData,
+    let start_ptr = self.as_mut_ptr();
+    unsafe {
+      intrinsics::assume(!is_null_mut(start_ptr));
+      StaticVecIterMut {
+        start: start_ptr,
+        end: match intrinsics::size_of::<T>() {
+          0 => (start_ptr as *mut u8).wrapping_add(self.length) as *mut T,
+          _ => start_ptr.add(self.length),
+        },
+        marker: PhantomData,
+      }
     }
   }
 
@@ -784,8 +803,8 @@ impl<T, const N: usize> StaticVec<T, N> {
   ///
   /// Unlike [`sorted`](crate::StaticVec::sorted) and
   /// [`sorted_unstable`](crate::StaticVec::sorted_unstable), this function does not make use of
-  /// Rust's built-in sorting methods, but instead makes direct use of a fairly unsophisticated
-  /// recursive quicksort algorithm implemented in this crate.
+  /// Rust's built-in sorting methods, but instead makes direct use of a comparatively
+  /// unsophisticated recursive quicksort algorithm implemented in this crate.
   ///
   /// This has the advantage of only needing to have [`PartialOrd`](core::cmp::PartialOrd) as a
   /// constraint as opposed to [`Ord`](core::cmp::Ord), but is very likely less performant for
@@ -801,12 +820,13 @@ impl<T, const N: usize> StaticVec<T, N> {
       return self.clone();
     }
     let mut res = Self::new_data_uninit();
+    let res_ptr = Self::first_ptr_mut(&mut res);
+    // Copy the inhabited part of `self` into the array we'll use for the returned StaticVec.
     unsafe {
-      self
-        .as_ptr()
-        .copy_to_nonoverlapping(Self::first_ptr_mut(&mut res), length);
+      self.as_ptr().copy_to_nonoverlapping(res_ptr, length);
     }
-    quicksort_internal(Self::first_ptr_mut(&mut res), 0, (length - 1) as isize);
+    // Sort the array, and then build and return a new StaticVec from it.
+    quicksort_internal(res_ptr, 0, (length - 1) as isize);
     Self { data: res, length }
   }
 
@@ -994,11 +1014,11 @@ impl<T, const N: usize> StaticVec<T, N> {
     T: Clone,
   {
     let mut res = StaticVec::new();
-    for i in 0..self.length {
-      unsafe { res.push_unchecked(self.get_unchecked(i).clone()) };
+    for item in self {
+      unsafe { res.push_unchecked(item.clone()) };
     }
-    for i in 0..other.length {
-      unsafe { res.push_unchecked(other.get_unchecked(i).clone()) };
+    for item in other {
+      unsafe { res.push_unchecked(item.clone()) };
     }
     res
   }
@@ -1084,6 +1104,7 @@ impl<T, const N: usize> StaticVec<T, N> {
     let item_count = vec_len.min(N);
     Self {
       data: {
+        // Set the length of `vec` to 0 to prevent double-drops.
         unsafe { vec.set_len(0) };
         let mut data = Self::new_data_uninit();
         unsafe {
@@ -1119,6 +1140,7 @@ impl<T, const N: usize> StaticVec<T, N> {
         .as_ptr()
         .copy_to_nonoverlapping(res.as_mut_ptr(), length);
       res.set_len(length);
+      // Set the length of `self` to 0 to prevent double-drops.
       self.set_len(0);
       res
     }
@@ -1261,9 +1283,9 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// and the new one will contain elements `at..self.len()`.
   #[inline]
   pub fn split_off(&mut self, at: usize) -> Self {
-    let length = self.length;
-    assert!(at <= length);
-    let split_length = length - at;
+    let old_length = self.length;
+    assert!(at <= old_length);
+    let split_length = old_length - at;
     Self {
       data: unsafe {
         self.set_len(at);
