@@ -1,6 +1,8 @@
+use crate::heap::StaticHeap;
 use crate::iterators::*;
+use crate::string::StaticString;
 use crate::utils::{partial_compare, slice_from_raw_parts, slice_from_raw_parts_mut};
-use crate::{StaticString, StaticVec};
+use crate::StaticVec;
 use core::borrow::{Borrow, BorrowMut};
 use core::cmp::{Eq, Ord, Ordering, PartialEq};
 use core::fmt::{self, Debug, Formatter};
@@ -161,9 +163,7 @@ impl<T, const N: usize> DerefMut for StaticVec<T, N> {
 impl<T, const N: usize> Drop for StaticVec<T, N> {
   #[inline(always)]
   fn drop(&mut self) {
-    unsafe {
-      ptr::drop_in_place(self.as_mut_slice());
-    }
+    unsafe { ptr::drop_in_place(self.as_mut_slice()) };
   }
 }
 
@@ -171,7 +171,7 @@ impl<T: Eq, const N: usize> Eq for StaticVec<T, N> {}
 
 /// A helper trait for specialization-based implementations of [`Extend`](core::iter::Extend) and
 /// ['FromIterator`](core::iter::FromIterator).
-trait ExtendEx<T, I> {
+pub(crate) trait ExtendEx<T, I> {
   fn extend_ex(&mut self, iter: I);
   fn from_iter_ex(iter: I) -> Self;
 }
@@ -352,6 +352,13 @@ impl<T: Copy, const N: usize> From<&mut [T; N]> for StaticVec<T, N> {
   }
 }
 
+impl<T, const N: usize> From<StaticHeap<T, N>> for StaticVec<T, N> {
+  #[inline(always)]
+  fn from(heap: StaticHeap<T, N>) -> StaticVec<T, N> {
+    heap.data
+  }
+}
+
 impl<const N1: usize, const N2: usize> From<StaticString<N1>> for StaticVec<u8, N2> {
   #[inline(always)]
   default fn from(string: StaticString<N1>) -> Self {
@@ -864,9 +871,7 @@ where T: Deserialize<'de>
         let mut res = Self::Value::new();
         while res.length < N {
           if let Some(val) = seq.next_element()? {
-            unsafe {
-              res.push_unchecked(val);
-            }
+            unsafe { res.push_unchecked(val) };
           } else {
             break;
           }
